@@ -15,7 +15,6 @@
 
 import functools
 import inspect
-import textwrap
 
 from twisted.internet import defer
 from twisted.python import reflect
@@ -23,7 +22,6 @@ from twisted.python import reflect
 from buildbot.data import base
 from buildbot.data import exceptions
 from buildbot.data import resultspec
-from buildbot.data.types import Entity
 from buildbot.util import bytes2unicode
 from buildbot.util import pathmatch
 from buildbot.util import service
@@ -81,7 +79,7 @@ class DataConnector(service.AsyncService):
                 rtype = obj(self.master)
                 setattr(self.rtypes, rtype.name, rtype)
                 setattr(self.plural_rtypes, rtype.plural, rtype)
-
+                self.graphql_rtypes[rtype.entityType.toGraphQLTypeName()] = rtype
                 # put its update methods into our 'updates' attribute
                 for name in dir(rtype):
                     o = getattr(rtype, name)
@@ -107,6 +105,7 @@ class DataConnector(service.AsyncService):
 
     def _setup(self):
         self.updates = Updates()
+        self.graphql_rtypes = {}
         self.rtypes = RTypes()
         self.plural_rtypes = RTypes()
         for moduleName in self.submodules:
@@ -121,7 +120,21 @@ class DataConnector(service.AsyncService):
                 "Invalid path: " + "/".join([str(p) for p in path])) from e
 
     def getResourceType(self, name):
-        return getattr(self.rtypes, name)
+        return getattr(self.rtypes, name, None)
+
+    def getEndPointForResourceName(self, name):
+        rtype = getattr(self.rtypes, name, None)
+        rtype_plural = getattr(self.plural_rtypes, name, None)
+        if rtype is not None:
+            return rtype.getDefaultEndpoint()
+        elif rtype_plural is not None:
+            return rtype_plural.getCollectionEndpoint()
+        return None
+
+    def getResourceTypeForGraphQlType(self, type):
+        if type not in self.graphql_rtypes:
+            raise RuntimeError(f"Can't get rtype for {type}: {self.graphql_rtypes.keys()}")
+        return self.graphql_rtypes.get(type)
 
     def get(self, path, filters=None, fields=None, order=None,
             limit=None, offset=None):
@@ -159,83 +172,6 @@ class DataConnector(service.AsyncService):
                               type_spec=v.rtype.entityType.getSpec()))
         return paths
 
-    @functools.lru_cache(1)
-    def get_graphql_schema(self):
-        """Return the graphQL Schema of the buildbot data model
-        """
-        types = {}
-        schema = textwrap.dedent("""
-        # custom scalar types for buildbot data model
-        scalar Date   # stored as utc unix timestamp
-        scalar Binary # arbitrary data stored as base85
-        scalar JSON  # arbitrary json stored as string, mainly used for properties values
-        """)
-
-        # type dependencies must be added recursively
-        def add_dependent_types(ent):
-            typename = ent.toGraphQLTypeName()
-            if typename not in types and isinstance(ent, Entity):
-                types[typename] = ent
-            for dtyp in ent.graphQLDependentTypes():
-                add_dependent_types(dtyp)
-
-        # root query contain the list of item available directly
-        # mapped against the rootLinks
-        schema += "type Query {\n"
-
-        def format_query_fields(query_fields):
-            query_fields = ",\n   ".join(query_fields)
-            if query_fields:
-                query_fields = f"({query_fields})"
-            return query_fields
-
-        operators = set(resultspec.Filter.singular_operators)
-        operators.update(resultspec.Filter.plural_operators)
-        for rootlink in sorted(v['name'] for v in self.rootLinks):
-            ep = self.matcher[(rootlink,)][0]
-            typ = ep.rtype.entityType
-            typename = typ.toGraphQLTypeName()
-            add_dependent_types(typ)
-            query_fields = []
-            # build the queriable parameters, via query_fields
-            for field in sorted(ep.rtype.entityType.fields.keys()):
-                field_type = ep.rtype.entityType.fields[field]
-                field_type_gql = field_type.getGraphQLInputType()
-                if field_type_gql is None:
-                    continue
-                query_fields.append(f"{field}: {field_type_gql}")
-                for op in sorted(operators):
-                    query_fields.append(f"{field}__{op}: {field_type_gql}")
-
-            query_fields.extend([
-                "order: String",
-                "limit: Int",
-                "offset: Int"]
-            )
-            schema += f"  {ep.rtype.plural}{format_query_fields(query_fields)}: [{typename}]!\n"
-
-            # build the queriable parameters, via keyFields
-            keyfields = []
-            for field in sorted(ep.rtype.keyFields):
-                field_type = ep.rtype.entityType.fields[field]
-                field_type_gql = field_type.toGraphQLTypeName()
-                keyfields.append(f"{field}: {field_type_gql}")
-
-            schema += f"  {ep.rtype.name}{format_query_fields(keyfields)}: {typename}\n"
-
-        schema += "}\n"
-
-        for name, typ in types.items():
-            type_spec = typ.toGraphQL()
-            schema += f"type {name} {{\n"
-            for field in type_spec.get('fields', []):
-                field_type = field['type']
-                if not isinstance(field_type, str):
-                    field_type = field_type['type']
-                schema += f"  {field['name']}: {field_type}\n"
-            schema += "}\n"
-        return schema
-
     def resultspec_from_jsonapi(self, req_args, entityType, is_collection):
 
         def checkFields(fields, negOk=False):
@@ -244,7 +180,7 @@ class DataConnector(service.AsyncService):
                 if k[0] == '-' and negOk:
                     k = k[1:]
                 if k not in entityType.fieldNames:
-                    raise exceptions.InvalidQueryParameter("no such field '{}'".format(k))
+                    raise exceptions.InvalidQueryParameter(f"no such field '{k}'")
 
         limit = offset = order = fields = None
         filters, properties = [], []
@@ -252,33 +188,32 @@ class DataConnector(service.AsyncService):
         filters, properties = [], []
         for arg in req_args:
             argStr = bytes2unicode(arg)
-            if arg == b'order':
-                order = tuple([bytes2unicode(o) for o in req_args[arg]])
+            if argStr == 'order':
+                order = tuple(bytes2unicode(o) for o in req_args[arg])
                 checkFields(order, True)
-            elif arg == b'field':
+            elif argStr == 'field':
                 fields = req_args[arg]
                 checkFields(fields, False)
-            elif arg == b'limit':
+            elif argStr == 'limit':
                 try:
                     limit = int(req_args[arg][0])
                 except Exception as e:
                     raise exceptions.InvalidQueryParameter('invalid limit') from e
-            elif arg == b'offset':
+            elif argStr == 'offset':
                 try:
                     offset = int(req_args[arg][0])
                 except Exception as e:
                     raise exceptions.InvalidQueryParameter('invalid offset') from e
-            elif arg == b'property':
+            elif argStr == 'property':
                 try:
                     props = []
                     for v in req_args[arg]:
                         if not isinstance(v, (bytes, str)):
-                            raise TypeError(
-                                "Invalid type {} for {}".format(type(v), v))
+                            raise TypeError(f"Invalid type {type(v)} for {v}")
                         props.append(bytes2unicode(v))
                 except Exception as e:
                     raise exceptions.InvalidQueryParameter(
-                        'invalid property value for {}'.format(arg)) from e
+                        f'invalid property value for {arg}') from e
                 properties.append(resultspec.Property(arg, 'eq', props))
             elif argStr in entityType.fieldNames:
                 field = entityType.fields[argStr]
@@ -286,7 +221,7 @@ class DataConnector(service.AsyncService):
                     values = [field.valueFromString(v) for v in req_args[arg]]
                 except Exception as e:
                     raise exceptions.InvalidQueryParameter(
-                        'invalid filter value for {}'.format(argStr)) from e
+                        f'invalid filter value for {argStr}') from e
 
                 filters.append(resultspec.Filter(argStr, 'eq', values))
             elif '__' in argStr:
@@ -302,11 +237,10 @@ class DataConnector(service.AsyncService):
                                   for v in req_args[arg]]
                     except Exception as e:
                         raise exceptions.InvalidQueryParameter(
-                            'invalid filter value for {}'.format(argStr)) from e
+                            f'invalid filter value for {argStr}') from e
                     filters.append(resultspec.Filter(field, op, values))
             else:
-                raise exceptions.InvalidQueryParameter(
-                    "unrecognized query parameter '{}'".format(argStr))
+                raise exceptions.InvalidQueryParameter(f"unrecognized query parameter '{argStr}'")
 
         # if ordering or filtering is on a field that's not in fields, bail out
         if fields:
