@@ -13,8 +13,6 @@
 #
 # Copyright Buildbot Team Members
 
-from __future__ import absolute_import
-from __future__ import print_function
 
 from buildbot.db import base
 
@@ -23,10 +21,11 @@ class TagsConnectorComponent(base.DBConnectorComponent):
 
     def findTagId(self, name):
         tbl = self.db.model.tags
+        name_hash = self.hashColumns(name)
         return self.findSomethingId(
             tbl=tbl,
-            whereclause=(tbl.c.name == name),
+            whereclause=(tbl.c.name_hash == name_hash),
             insert_values=dict(
                 name=name,
-                name_hash=self.hashColumns(name),
+                name_hash=name_hash,
             ))
