@@ -71,7 +71,10 @@ class Build(properties.PropertiesMixin):
     set_runtime_properties = True
     subs = None
 
-    _sentinel = []  # used as a sentinel to indicate unspecified initial_value
+    class Sentinel:
+        pass
+
+    _sentinel = Sentinel()  # used as a sentinel to indicate unspecified initial_value
 
     def __init__(self, requests):
         self.requests = requests
@@ -157,8 +160,8 @@ class Build(properties.PropertiesMixin):
         return files
 
     def __repr__(self):
-        return "<Build {} number:{} results:{}>".format(self.builder.name, repr(self.number),
-                                                        statusToString(self.results))
+        return (f"<Build {self.builder.name} number:{repr(self.number)} "
+                f"results:{statusToString(self.results)}>")
 
     def blamelist(self):
         # Note that this algorithm is also implemented in
@@ -197,11 +200,11 @@ class Build(properties.PropertiesMixin):
         return self.workerforbuilder.getWorkerCommandVersion(command, oldversion)
 
     def getWorkerName(self):
-        return self.workerforbuilder.worker.workername
+        return self.workername
 
     @staticmethod
     def setupPropertiesKnownBeforeBuildStarts(props, requests, builder,
-                                              workerforbuilder):
+                                              workerforbuilder=None):
         # Note that this function does not setup the 'builddir' worker property
         # It's not possible to know it until before the actual worker has
         # attached.
@@ -225,17 +228,21 @@ class Build(properties.PropertiesMixin):
         # get worker properties
         # navigate our way back to the L{buildbot.worker.Worker}
         # object that came from the config, and get its properties
-        workerforbuilder.worker.setupProperties(props)
+        if workerforbuilder is not None:
+            workerforbuilder.worker.setupProperties(props)
 
-    def setupOwnProperties(self):
+    @staticmethod
+    def setupBuildProperties(props, requests, sources=None, number=None):
         # now set some properties of our own, corresponding to the
         # build itself
-        props = self.getProperties()
-        props.setProperty("buildnumber", self.number, "Build")
+        props.setProperty("buildnumber", number, "Build")
 
-        if self.sources and len(self.sources) == 1:
+        if sources is None:
+            sources = requests[0].mergeSourceStampsWith(requests[1:])
+
+        if sources and len(sources) == 1:
             # old interface for backwards compatibility
-            source = self.sources[0]
+            source = sources[0]
             props.setProperty("branch", source.branch, "Build")
             props.setProperty("revision", source.revision, "Build")
             props.setProperty("repository", source.repository, "Build")
@@ -256,6 +263,7 @@ class Build(properties.PropertiesMixin):
     def setupWorkerForBuilder(self, workerforbuilder):
         self.path_module = workerforbuilder.worker.path_module
         self.workername = workerforbuilder.worker.workername
+        self.worker_info = workerforbuilder.worker.info
 
     @defer.inlineCallbacks
     def getBuilderId(self):
@@ -290,7 +298,12 @@ class Build(properties.PropertiesMixin):
 
         worker = workerforbuilder.worker
 
-        log.msg("{}.startBuild".format(self))
+        # Cache the worker information as variables instead of accessing via worker, as the worker
+        # will disappear during disconnection and some of these properties may still be needed.
+        self.workername = worker.workername
+        self.worker_info = worker.info
+
+        log.msg(f"{self}.startBuild")
 
         # TODO: this will go away when build collapsing is implemented; until
         # then we just assign the build to the first buildrequest
@@ -314,10 +327,11 @@ class Build(properties.PropertiesMixin):
         self.preparation_step = buildstep.BuildStep(name="worker_preparation")
         self.preparation_step.setBuild(self)
         yield self.preparation_step.addStep()
-        self.setupOwnProperties()
+        Build.setupBuildProperties(self.getProperties(), self.requests,
+                                   self.sources, self.number)
 
         # then narrow WorkerLocks down to the right worker
-        self.locks = [(l.getLockForWorker(workerforbuilder.worker.workername),
+        self.locks = [(l.getLockForWorker(self.workername),
                        a)
                       for l, a in self.locks]
         metrics.MetricCountEvent.log('active_builds', 1)
@@ -372,7 +386,7 @@ class Build(properties.PropertiesMixin):
         # us in a build.
         yield self.master.data.updates.setBuildStateString(self.buildid,
                                                            'pinging worker')
-        log.msg("starting build {}.. pinging the worker {}".format(self, workerforbuilder))
+        log.msg(f"starting build {self}.. pinging the worker {workerforbuilder}")
         try:
             ping_success_or_failure = yield workerforbuilder.ping()
         except Exception:
@@ -405,7 +419,7 @@ class Build(properties.PropertiesMixin):
                                                            'acquiring locks')
         yield self.acquireLocks()
 
-        readymsg = "worker {} ready".format(self.getWorkerName())
+        readymsg = f"worker {self.getWorkerName()} ready"
         yield self.master.data.updates.setStepStateString(self.preparation_step.stepid, readymsg)
         yield self.master.data.updates.finishStep(self.preparation_step.stepid, SUCCESS, False)
 
@@ -448,10 +462,10 @@ class Build(properties.PropertiesMixin):
             return defer.succeed(None)
         if self.stopped:
             return defer.succeed(None)
-        log.msg("acquireLocks(build {}, locks {})".format(self, self.locks))
+        log.msg(f"acquireLocks(build {self}, locks {self.locks})")
         for lock, access in self.locks:
             if not lock.isAvailable(self, access):
-                log.msg("Build {} waiting for lock {}".format(self, lock))
+                log.msg(f"Build {self} waiting for lock {lock}")
                 d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 self._acquiringLock = (lock, access, d)
@@ -469,7 +483,7 @@ class Build(properties.PropertiesMixin):
             count = self.stepnames[name]
             count += 1
             self.stepnames[name] = count
-            name = "{}_{}".format(step.name, count)
+            name = f"{step.name}_{count}"
         else:
             self.stepnames[name] = 0
         step.name = name
@@ -557,7 +571,7 @@ class Build(properties.PropertiesMixin):
             yield self.startNextStep()
 
         except Exception as e:
-            log.msg("{} build got exception when running step {}".format(self, step))
+            log.msg(f"{self} build got exception when running step {step}")
             log.err(e)
 
             yield self.master.data.updates.setBuildProperties(self.buildid, self)
@@ -575,11 +589,11 @@ class Build(properties.PropertiesMixin):
         text = None
         if isinstance(results, tuple):
             results, text = results
-        assert isinstance(results, type(SUCCESS)), "got %r" % (results,)
+        assert isinstance(results, type(SUCCESS)), f"got {repr(results)}"
         summary = yield step.getBuildResultSummary()
         if 'build' in summary:
             text = [summary['build']]
-        log.msg(" step '{}' complete: {} ({})".format(step.name, statusToString(results), text))
+        log.msg(f" step '{step.name}' complete: {statusToString(results)} ({text})")
         if text:
             self.text.extend(text)
             self.master.data.updates.setBuildStateString(self.buildid,
@@ -596,7 +610,7 @@ class Build(properties.PropertiesMixin):
         # the worker went away. There are several possible reasons for this,
         # and they aren't necessarily fatal. For now, kill the build, but
         # TODO: see if we can resume the build when it reconnects.
-        log.msg("{}.lostRemote".format(self))
+        log.msg(f"{self}.lostRemote")
         self.conn = None
         self.text = ["lost", "connection"]
         self.results = RETRY
@@ -622,7 +636,7 @@ class Build(properties.PropertiesMixin):
         # build as failed quickly rather than waiting for the worker's
         # timeout to kill it on its own.
 
-        log.msg(" {}: stopping build: {} {}".format(self, reason, results))
+        log.msg(f" {self}: stopping build: {reason} {results}")
         if self.finished:
             return
         # TODO: include 'reason' in this point event
@@ -677,7 +691,7 @@ class Build(properties.PropertiesMixin):
                 self.subs.unsubscribe()
                 self.subs = None
                 self.conn = None
-            log.msg(" {}: build finished".format(self))
+            log.msg(f" {self}: build finished")
             self.results = worst_status(self.results, results)
             eventually(self.releaseLocks)
             metrics.MetricCountEvent.log('active_builds', -1)
@@ -708,7 +722,7 @@ class Build(properties.PropertiesMixin):
 
     def releaseLocks(self):
         if self.locks:
-            log.msg("releaseLocks({}): {}".format(self, self.locks))
+            log.msg(f"releaseLocks({self}): {self.locks}")
 
         for lock, access in self.locks:
             if lock.isOwner(self, access):
@@ -770,4 +784,4 @@ class Build(properties.PropertiesMixin):
                                             lambda: self.finished)
 
     def getWorkerInfo(self):
-        return self.workerforbuilder.worker.info
+        return self.worker_info
