@@ -22,10 +22,13 @@ from twisted.internet import defer
 from twisted.internet import reactor
 
 from buildbot import config
+from buildbot.data.graphql import GraphQLConnector
 from buildbot.test import fakedb
 from buildbot.test.fake import bworkermanager
+from buildbot.test.fake import endpoint
 from buildbot.test.fake import fakedata
 from buildbot.test.fake import fakemq
+from buildbot.test.fake import msgmanager
 from buildbot.test.fake import pbmanager
 from buildbot.test.fake.botmaster import FakeBotMaster
 from buildbot.test.fake.machine import FakeMachineManager
@@ -90,7 +93,7 @@ class FakeMaster(service.MasterService):
         self._master_id = master_id
         self.reactor = reactor
         self.objectids = {}
-        self.config = config.MasterConfig()
+        self.config = config.master.MasterConfig()
         self.caches = FakeCaches()
         self.pbmanager = pbmanager.FakePBManager()
         self.initLock = defer.DeferredLock()
@@ -99,6 +102,7 @@ class FakeMaster(service.MasterService):
         self.botmaster.setServiceParent(self)
         self.name = 'fake:/master'
         self.masterid = master_id
+        self.msgmanager = msgmanager.FakeMsgManager()
         self.workers = bworkermanager.FakeWorkerManager()
         self.workers.setServiceParent(self)
         self.machine_manager = FakeMachineManager()
@@ -128,7 +132,7 @@ class FakeMaster(service.MasterService):
 
 
 def make_master(testcase, wantMq=False, wantDb=False, wantData=False,
-                wantRealReactor=False, url=None, **kwargs):
+                wantRealReactor=False, wantGraphql=False, url=None, **kwargs):
     if wantRealReactor:
         _reactor = reactor
     else:
@@ -151,4 +155,14 @@ def make_master(testcase, wantMq=False, wantDb=False, wantData=False,
         master.db.setServiceParent(master)
     if wantData:
         master.data = fakedata.FakeDataConnector(master, testcase)
+    if wantGraphql:
+        master.graphql = GraphQLConnector()
+        master.graphql.setServiceParent(master)
+        master.graphql.data = master.data.realConnector
+        master.data._scanModule(endpoint)
+        master.config.www = {'graphql': {"debug": True}}
+        try:
+            master.graphql.reconfigServiceWithBuildbotConfig(master.config)
+        except ImportError:
+            pass
     return master
