@@ -19,13 +19,14 @@ import textwrap
 
 import mock
 
+from autobahn.wamp.exception import TransportLost
 from autobahn.wamp.types import SubscribeOptions
 from twisted.internet import defer
 from twisted.trial import unittest
 
 from buildbot.mq import wamp
 from buildbot.test.fake import fakemaster
-from buildbot.test.util.misc import TestReactorMixin
+from buildbot.test.reactor import TestReactorMixin
 from buildbot.wamp import connector
 
 
@@ -44,15 +45,30 @@ class ComparableSubscribeOptions(SubscribeOptions):
     __repr__ = SubscribeOptions.__str__
 
 
+class FakeSubscription:
+    def __init__(self):
+        self.exception_on_unsubscribe = None
+
+    def unsubscribe(self):
+        if self.exception_on_unsubscribe is not None:
+            raise self.exception_on_unsubscribe()
+
+
+class TestException(Exception):
+    pass
+
+
 class FakeWampConnector:
     # a fake wamp connector with only one queue
+    def __init__(self):
+        self.subscriptions = []
 
     def topic_match(self, topic):
         topic = topic.split(".")
         owntopic = self.topic.split(".")
         if len(topic) != len(owntopic):
             return False
-        for i, topic_item in enumerate(topic):
+        for i, _ in enumerate(topic):
             if owntopic[i] != "" and topic[i] != owntopic[i]:
                 return False
         return True
@@ -64,6 +80,10 @@ class FakeWampConnector:
         # we record the qref_cb
         self.qref_cb = callback
 
+        subs = FakeSubscription()
+        self.subscriptions.append(subs)
+        return subs
+
     def publish(self, topic, data, options=None):
         # make sure the topic is compatible with what was subscribed
         assert self.topic_match(topic)
@@ -105,12 +125,19 @@ class WampMQ(TestReactorMixin, unittest.TestCase):
         A router which only accepts one subscriber on one topic
     """
 
+    @defer.inlineCallbacks
     def setUp(self):
-        self.setUpTestReactor()
+        self.setup_test_reactor()
         self.master = fakemaster.make_master(self)
         self.master.wamp = FakeWampConnector()
         self.mq = wamp.WampMQ()
-        self.mq.setServiceParent(self.master)
+        yield self.mq.setServiceParent(self.master)
+        yield self.mq.startService()
+
+    @defer.inlineCallbacks
+    def tearDown(self):
+        if self.mq.running:
+            yield self.mq.stopService()
 
     @defer.inlineCallbacks
     def test_startConsuming_basic(self):
@@ -140,6 +167,28 @@ class WampMQ(TestReactorMixin, unittest.TestCase):
         callback.assert_called_with(('a', 'b'), 'foo')
         self.assertEqual(self.master.wamp.last_data, 'foo')
 
+    @defer.inlineCallbacks
+    def test_unsubscribe_ignores_transport_lost(self):
+        callback = mock.Mock()
+        consumer = yield self.mq.startConsuming(callback, ('a', 'b'))
+
+        self.assertEqual(len(self.master.wamp.subscriptions), 1)
+        self.master.wamp.subscriptions[0].exception_on_unsubscribe = TransportLost
+
+        yield consumer.stopConsuming()
+
+    @defer.inlineCallbacks
+    def test_unsubscribe_logs_exceptions(self):
+        callback = mock.Mock()
+        consumer = yield self.mq.startConsuming(callback, ('a', 'b'))
+
+        self.assertEqual(len(self.master.wamp.subscriptions), 1)
+        self.master.wamp.subscriptions[0].exception_on_unsubscribe = TestException
+
+        yield consumer.stopConsuming()
+
+        self.assertEqual(len(self.flushLoggedErrors(TestException)), 1)
+
     @defer.inlineCallbacks
     def test_forward_data_wildcard(self):
         callback = mock.Mock()
@@ -151,6 +200,34 @@ class WampMQ(TestReactorMixin, unittest.TestCase):
         callback.assert_called_with(('a', 'b'), 'foo')
         self.assertEqual(self.master.wamp.last_data, 'foo')
 
+    @defer.inlineCallbacks
+    def test_waits_for_called_callback(self):
+        def callback(_, __):
+            return defer.succeed(None)
+
+        yield self.mq.startConsuming(callback, ('a', None))
+        yield self.mq._produce(('a', 'b'), 'foo')
+        self.assertEqual(self.master.wamp.last_data, 'foo')
+
+        d = self.mq.stopService()
+        self.assertTrue(d.called)
+
+    @defer.inlineCallbacks
+    def test_waits_for_non_called_callback(self):
+        d1 = defer.Deferred()
+
+        def callback(_, __):
+            return d1
+
+        yield self.mq.startConsuming(callback, ('a', None))
+        yield self.mq._produce(('a', 'b'), 'foo')
+        self.assertEqual(self.master.wamp.last_data, 'foo')
+
+        d = self.mq.stopService()
+        self.assertFalse(d.called)
+        d1.callback(None)
+        self.assertTrue(d.called)
+
 
 class FakeConfig:
     mq = dict(type='wamp', router_url="wss://foo", realm="realm1")
@@ -174,7 +251,7 @@ class WampMQReal(TestReactorMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
-        self.setUpTestReactor()
+        self.setup_test_reactor()
         if "WAMP_ROUTER_URL" not in os.environ:
             raise unittest.SkipTest(self.HOW_TO_RUN)
         self.master = fakemaster.make_master(self)
