@@ -5,6 +5,7 @@ from buildbot.process.properties import Secret
 from buildbot.secrets.manager import SecretManager
 from buildbot.test.fake import fakemaster
 from buildbot.test.fake.secrets import FakeSecretStorage
+from buildbot.test.reactor import TestReactorMixin
 from buildbot.util.service import BuildbotService
 
 
@@ -19,24 +20,23 @@ class FakeServiceUsingSecrets(BuildbotService):
         self.secret = secret
 
     def returnRenderedSecrets(self, secretKey):
-        try:
-            return getattr(self, secretKey)
-        except Exception:
-            raise Exception
+        return getattr(self, secretKey)
 
 
-class TestRenderSecrets(unittest.TestCase):
+class TestRenderSecrets(TestReactorMixin, unittest.TestCase):
 
+    @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master()
+        self.setup_test_reactor()
+        self.master = fakemaster.make_master(self)
         fakeStorageService = FakeSecretStorage(secretdict={"foo": "bar",
                                                        "other": "value"})
         self.secretsrv = SecretManager()
         self.secretsrv.services = [fakeStorageService]
-        self.secretsrv.setServiceParent(self.master)
+        yield self.secretsrv.setServiceParent(self.master)
         self.srvtest = FakeServiceUsingSecrets()
-        self.srvtest.setServiceParent(self.master)
-        self.successResultOf(self.master.startService())
+        yield self.srvtest.setServiceParent(self.master)
+        yield self.master.startService()
 
     @defer.inlineCallbacks
     def tearDown(self):
