@@ -37,7 +37,7 @@ class GitoriousHandler(BaseHookHandler):
         project = payload['project']['name']
 
         changes = self.process_change(payload, user, repo, repo_url, project)
-        log.msg("Received {} changes from gitorious".format(len(changes)))
+        log.msg(f"Received {len(changes)} changes from gitorious")
         return (changes, 'git')
 
     def process_change(self, payload, user, repo, repo_url, project):
@@ -46,7 +46,7 @@ class GitoriousHandler(BaseHookHandler):
 
         branch = payload['ref']
         if re.match(r"^0*$", newrev):
-            log.msg("Branch `{}' deleted, ignoring".format(branch))
+            log.msg(f"Branch `{branch}' deleted, ignoring")
             return []
         else:
             for commit in payload['commits']:
@@ -60,9 +60,9 @@ class GitoriousHandler(BaseHookHandler):
                 #     files.extend(commit['removed'])
                 when_timestamp = dateparse(commit['timestamp'])
 
-                log.msg("New revision: {}".format(commit['id'][:8]))
+                log.msg(f"New revision: {commit['id'][:8]}")
                 changes.append({
-                    'author': '{} <{}>'.format(commit['author']['name'], commit['author']['email']),
+                    'author': f"{commit['author']['name']} <{commit['author']['email']}>",
                     'files': files,
                     'comments': commit['message'],
                     'revision': commit['id'],
